#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TAM 10
#define MAX 60

typedef void* PVOID;
typedef void** PPVOID;

struct lista {
	PPVOID cabeca;
	int tamanho;
	int posicao;
};
typedef struct lista LISTA;
typedef LISTA* PLISTA;

struct aluno {
	int numero;
	char nome[MAX];
};
typedef struct aluno ALUNO;
typedef ALUNO* PALUNO;

/* Cria aluno */
PALUNO criaAluno() {
        PALUNO novo;
        novo = (PALUNO)malloc(sizeof(ALUNO));
        printf("Nmero: ");
        scanf("%d", &novo->numero);
	/* Apaga resto da linha */
        scanf("%*[^\n]"); scanf("%*c");
        printf("Nome: ");
        fflush (stdout);
        fgets(novo->nome, MAX, stdin);
	/* Elimina '\n' no fim da string */
        novo->nome[strlen(novo->nome)-1] = '\0';
        return novo;
}

/* Inicializa a lista */
void initLista(PLISTA l) {
	l->cabeca = (PPVOID)malloc(sizeof(PVOID) * TAM);
	l->tamanho = TAM;
	l->posicao = 0;
}

/* Insere aluno no fim do vetor */
void insere(PLISTA l, PVOID dados) {
	if(l->posicao >= l->tamanho) {
		l->cabeca = (PPVOID)realloc(l->cabeca, sizeof(PVOID) * (l->tamanho + TAM));
		l->tamanho += TAM;
	}
	l->cabeca[l->posicao] = dados;
	l->posicao++;
}

void imprime(PLISTA l, void (*print)()) {
	int i;
	for(i=0; i<l->posicao; i++)
		(*print)(l->cabeca[i]);
}

void imprimeAluno(PALUNO a) {
	printf("%d - %s\n", a->numero, a->nome);
}

/* Funo de pesquisa */
void* pesquisa(PLISTA l, int (*compara)(), void* valor) {
	int i;
	for(i=0; i<l->posicao; i++)
		if((*compara)(l->cabeca[i], valor))
			return l->cabeca[i];
	return NULL;
}

/* Funes auxiliares de pesquisa */
int cmpNumAluno(PALUNO a, int* num) {
	return a->numero == *num;
}

int cmpNomeAluno(PALUNO a, char* nome) {
	return strcmp(a->nome, nome) == 0;
}

/* Ordenao - Bubblesort */
void bubblesort(PLISTA l, int (*compara)()){
	int i=0, j, trocou;
	PVOID t;
	do {
		trocou = 0;
		for(j=0; j<l->posicao-1-i; j++)	/* Percorre vetor */
			if( (*compara)(l->cabeca[j], l->cabeca[j+1]) ) {
				t = l->cabeca[j];
				l->cabeca[j] = l->cabeca[j+1];
				l->cabeca[j+1] = t;
				trocou = 1;
			}
		i++;
	} while(trocou==1);
}

/* Ordenao - funes auxiliares */
int nomeCres(PALUNO a, PALUNO b) {
	return strcmp(a->nome, b->nome) > 0;
}

int nomeDecr(PALUNO a, PALUNO b) {
	return strcmp(a->nome, b->nome) < 0;
}

int numCres(PALUNO a, PALUNO b) {
	return a->numero > b->numero;
}

int numDecr(PALUNO a, PALUNO b) {
	return a->numero < b->numero;
}

/* Funo auxiliar de remocao */
void removeIndice(PLISTA l, int p) {
	int i;
	free(l->cabeca[p]);
	for(i=p; i<l->posicao-1;i++)
		l->cabeca[i] = l->cabeca[i+1];
	l->posicao--;
}

/* Funo de remocao */
int remover(PLISTA l, int (*compara)(), void* valor) {
        int i;
        for(i=0; i<l->posicao; i++)
                if((*compara)(l->cabeca[i], valor)) {
			removeIndice(l,i);
                        return 1;
		}
	return 0;
}

int main() {
	PALUNO a;
	LISTA turma;
	initLista(&turma);
	int num, n;
	char nome[MAX];
	int opcao;
	do {
		printf("\nVetores Dinmicos\n");
		printf("\n1 - Inserir aluno");
		printf("\n2 - Imprimir lista de alunos");
		printf("\n3 - Pesquisar por nmero");
		printf("\n4 - Pesquisar por nome");
		printf("\n5 - Ordenar por nmero, ordem crescente");
		printf("\n6 - Ordenar por nome, ordem crescente");
		printf("\n7 - Ordenar por nmero, ordem decrescente");
		printf("\n8 - Ordenar por nome, ordem decrescente");
		printf("\n9 - Remover por nmero");
		printf("\na - Remover por nome");
		printf("\n0 - Sair");
		printf("\nInsira a opo: ");
		opcao = getchar();
		scanf("%*[^\n]"); scanf("%*c");
		switch(opcao) {
			case '1':
				a = criaAluno();
				insere(&turma,a);
				break;
			case '2':
				imprime(&turma,imprimeAluno);
				break;
			case '3':
				printf("Numero: ");
			        scanf("%d", &num);
 			        scanf("%*[^\n]"); scanf("%*c");
				a = pesquisa(&turma, cmpNumAluno, &num);
				if(a) imprimeAluno(a);
				else printf("Aluno inexistente");
				break;
			case '4':
				printf("Nome: ");
			        fgets(nome, MAX, stdin);
			        nome[strlen(nome)-1] = '\0';
				a = pesquisa(&turma, cmpNomeAluno, nome);
				if(a) imprimeAluno(a);
				else printf("Aluno inexistente");
				break;
			case '5':
				bubblesort(&turma, numCres);
				break;
			case '6':
				bubblesort(&turma, nomeCres);
				break;
			case '7':
				bubblesort(&turma, numDecr);
				break;
			case '8':
				bubblesort(&turma, nomeDecr);
				break;
			case '9':
				printf("Numero: ");
			        scanf("%d", &num);
 			        scanf("%*[^\n]"); scanf("%*c");
				n = remover(&turma, cmpNumAluno, &num);
				if(n) printf("Aluno removido");
				else printf("Aluno inexistente");
				break;
			case 'a':
				printf("Nome: ");
			        fgets(nome, MAX, stdin);
			        nome[strlen(nome)-1] = '\0';
				n = remover(&turma, cmpNomeAluno, nome);
				if(n) printf("Aluno removido");
				else printf("Aluno inexistente");
				break;
		}
	} while(opcao != '0');
}

